#!/usr/bin/python


#-----------------------------------------------------------------------
#  About
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
#  Imports
#-----------------------------------------------------------------------

import os
import sys
import gzip
import datetime

#-----------------------------------------------------------------------
#  Constants
#-----------------------------------------------------------------------

LOG_SUB_DIR = 'data/reports/30min/port'
LOG_SUFFIX  = '.port.log'

#  Number of minutes between reporting periods
REPORTING_PERIOD = 30

#  How many top ports to log for each ipaudit data file?
NTOP_DATA_FILE = 20

#  How many top ports to plot.
#    Plot the NTOP_PLOT port counts from the past NTOP_PLOT_HISTORY 
#    minutes.
NTOP_PLOT =  6

#  From how many minutes past should we search for top ports
#  to determine which NTOP_PLOT will be plotted?
NTOP_PLOT_HISTORY = 24*60

#  Should we plot using logarithmic scale?
USE_LOG_SCALE = 0

#  Where should we write the image files?
IMG_SUB_DIR = "public_html/images"

#  Data directory
DATA_SUB_DIR = "data/30min"

#  Maximum number of files to process in one call to this script
MAX_FILES_PROCESS = 10

#  Only used the first time when no logs exist
#  Determine how many days back to start accumulating port count data.
#  If logs already exist, then we start accumulating data for the
#  first period after the log end.
DEF_START_INTERVAL = 7

#-----------------------------------------------------------------------
#  Function
#-----------------------------------------------------------------------

#  Scan this script's own port logs, ignoring comments (# delimited) and
#  blank lines and return the value of the first space delimited field
#  with the maximum alpha-numeric value.  This will be the latest
#  datetime in the log file.
def getmaxfromlog (logdir,logsuffix):
	#  Intialize max filed found
	max_field=""
	#  Read last datetime from this script's own port logs
	try:
		#  Get sorted list of this script's port logs
		files = filter(lambda x: x[-len(logsuffix):]==logsuffix, os.listdir(logdir))
		files.sort()
		#  Open port logs and search for max field
		for file in files:
			#  Read port count log file
			f = open(logdir+"/"+file)
			#  Read log lines
			for line in f.readlines():
				#  Get first field in record
				try:
					this_field = line.split()[0]
					if this_field[0]=="#":
						continue
				except:
					#  Skip blank lines
					continue
				#  Keep this field if the current maximum
				if max_field=="" or max_field < this_field:
					max_field = this_field
			f.close()
	except:
		pass
	return max_field


#  Read ipaudit data log and look at incoming scans (connections with
#  now outgoing bytes).  Count the number of such connections for every
#  protocol/local port combination and return the top NTOP_DATA_FILE 
#  for each ipaudit data file.
def count_ports(path):
	#  Open zip file
	if path[-3:]==".gz":
		#  Read zip file
		f = gzip.open(path)
	#  Open text file
	else:
		f = open(path)

	#  Tabulate
	portcount = {}
	for line in f.readlines():
		#  Need only first 7 fields
		fields = line.split(" ",7)
		#  Connection must be incoming only
		if not fields[6]=="0":
			continue
		#  Make key from TCP port
		if fields[2]=="6":
			port = fields[3]+"t"
		#  Make port from UDP port
		elif fields[2]=="17":
			port = fields[3]+"u"
		#  This connection not TCP or UDP
		else:
			continue
		#  Count hits to this port
		if portcount.has_key(port):
			portcount[port] = portcount[port] + 1
		else:
			portcount[port] = 1

	#  Get keys to portscan and sort in descending order of portscan count
	ports = portcount.keys()
	ports.sort(lambda x,y: cmp(portcount[y],portcount[x]))
	top_port_count = []
	topcnt = 0
	for port in ports:
		top_port_count.append( (port,portcount[port]) )
		topcnt = topcnt + 1
		if topcnt>=NTOP_DATA_FILE:
			break
	return top_port_count


#  Write port name and port hit count into a daily log
def writeportlog(logsubdir,logsuffix,datetime,portcount):
	date = datetime[0:10]
	logdir = HOME + "/" + logsubdir
	filepath = logdir + "/" + date + logsuffix
	if not os.path.isdir(logdir):
		os.makedirs(logdir)
	f = open(filepath,"a")
	for port,cnt in portcount:
		f.write(datetime + " " + port + " " + str(cnt) + "\n")
	f.close()
		
	

#-----------------------------------------------------------------------
#  Main
#-----------------------------------------------------------------------


#  Get home directory
try:
	HOME = sys.argv[1] 
except:
	HOME = "/var/tmp"


#  Read max datetime from this script's own port count logs
min_datetime = getmaxfromlog(HOME+"/"+LOG_SUB_DIR,LOG_SUFFIX)

#  Don't go further back in time than the default min_datetime.
def_min_datetime = str(datetime.date.today() - datetime.timedelta(DEF_START_INTERVAL))
if min_datetime<def_min_datetime:
	min_datetime = def_min_datetime

#
#  Find files in ipaudit data directory older then min_datetime 
#  as recorded in their file names.
#
DATA_DIR = HOME + "/" + DATA_SUB_DIR
datetimes = {}
for file in os.listdir(DATA_DIR):
	path = DATA_DIR+"/"+file
	file_datetime = file[0:len(min_datetime)]
	if os.path.isfile(path) and file_datetime > min_datetime:
		datetimestr = file[0:16]
		datetimes[path] = datetimestr
paths = datetimes.keys()
paths.sort()

#
#  Get port scan info from ipaudit data logs.
#
filecnt=0
for path in paths:
	#  portlist is a list of port/protocol and counts, like this
	#      10000t  25004
	#      34424u  10495
	portcounts = count_ports(path)
	#  Write portlist info to log
	writeportlog(LOG_SUB_DIR,LOG_SUFFIX,datetimes[path],portcounts)
	#  Stop if we've processed the maximum number of files.
	filecnt = filecnt + 1
	if filecnt>=MAX_FILES_PROCESS:
		break

#  Data collecting done.
#  Do plotting here, or maybe do that in a separate program.
